from ..schemas import LatestEarthquake, Shakemap
from ..types import InfoLatestEarthquakeData
from .parse_earthquake_data import parse_earthquake_data

__all__ = ["parse_latest_earthquake_data"]


def parse_latest_earthquake_data(
    info_latest_earthquake_data: InfoLatestEarthquakeData,
) -> LatestEarthquake:
    """
    Parse `InfoLatestEarthquakeData` JSON.

    Args:
        info_latest_earthquake_data: A dict representing JSON of info latest earthquake.

    Returns:
        A `LatestEarthquake` schema.

    Examples:
        >>> info_latest_earthquake_data = InfoLatestEarthquakeData(
        ...     {
        ...         "Infogempa": {
        ...             "gempa": {
        ...                 "Tanggal": "15 Jan 2024",
        ...                 "Jam": "16:07:08 WIB",
        ...                 "DateTime": "2024-01-15T09:07:08+00:00",
        ...                 "Coordinates": "-3.98,122.45",
        ...                 "Lintang": "3.98 LS",
        ...                 "Bujur": "122.45 BT",
        ...                 "Magnitude": "2.6",
        ...                 "Kedalaman": "9 km",
        ...                 "Wilayah": "Pusat gempa berada di darat 2,5 km BaratDaya P",
        ...                 "Potensi": "Gempa ini dirasakan untuk diteruskan pada masy",
        ...                 "Dirasakan": "II Kendari",
        ...                 "Shakemap": "20240115160708.mmi.jpg",
        ...             }
        ...         }
        ...     }
        ... )
        >>> parse_latest_earthquake_data(info_latest_earthquake_data)
        LatestEarthquake(earthquake=Earthquake(datetime=datetime.datetime(2024, 1, 15...
    """

    latest_earthquake_data = info_latest_earthquake_data["Infogempa"]["gempa"]
    earthquake = parse_earthquake_data(latest_earthquake_data)
    potency = latest_earthquake_data["Potensi"]
    felt = latest_earthquake_data["Dirasakan"]
    shakemap = latest_earthquake_data["Shakemap"]

    return LatestEarthquake(earthquake, potency, felt, Shakemap(shakemap))



"""
Module to store an int enum class representation of the weather code.
"""
from enum import IntEnum

__all__ = ["Weather"]


class Weather(IntEnum):
    """
    An int enum class that define valid weather.

    Attributes:
        CLEAR_SKIES: `0`
        PARTLY_CLOUDY: `1`
        PARTLY_CLOUDY2: `2`
        MOSTLY_CLOUDY: `3`
        OVERCAST: `4`
        HAZE: `5`
        SMOKE: `10`
        FOG: `45`
        LIGHT_RAIN: `60`
        RAIN: `61`
        HEAVY_RAIN: `63`
        ISOLATED_SHOWER: `80`
        SEVERE_THUNDERSTORM: `95`
        SEVERE_THUNDERSTORM2: `97`

    Examples:
        >>> Weather(0)
        <Weather.CLEAR_SKIES: 0>
        >>> Weather["CLEAR_SKIES"]
        <Weather.CLEAR_SKIES: 0>
        >>> Weather.CLEAR_SKIES
        <Weather.CLEAR_SKIES: 0>
        >>> Weather.CLEAR_SKIES == 0
        True
        >>> print(Weather.CLEAR_SKIES)
        0

    Note:
        There is `PARTLY_CLOUDY` and `PARTLY_CLOUDY2`, the weather
        condition is equal only the number representation is different. This is
        also hold true for `SEVERE_THUNDERSTORM` and `SEVERE_THUNDERSTORM2`.
    """

    CLEAR_SKIES: int = 0
    PARTLY_CLOUDY: int = 1
    PARTLY_CLOUDY2: int = 2
    MOSTLY_CLOUDY: int = 3
    OVERCAST: int = 4
    HAZE: int = 5
    SMOKE: int = 10
    FOG: int = 45
    LIGHT_RAIN: int = 60
    RAIN: int = 61
    HEAVY_RAIN: int = 63
    ISOLATED_SHOWER: int = 80
    SEVERE_THUNDERSTORM: int = 95
    SEVERE_THUNDERSTORM2: int = 97



from datetime import datetime

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError

__all__ = ["parse_issue_element"]


def parse_issue_element(element: Element) -> datetime:
    """
    Parse issue element tree in xml.

    Args:
        element: an element of issue

    Returns:
        A naive `datetime` object.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     "<issue>"
    ...     "<timestamp>20240116032347</timestamp>"
    ...     "<year>2024</year>"
    ...     "<month>01</month>"
    ...     "<day>16</day>"
    ...     "<hour>03</hour>"
    ...     "<minute>23</minute>"
    ...     "<second>47</second>"
    ...     "</issue>"
    ... )
    >>> issue = parse_issue_element(element)
    >>> issue
    datetime.datetime(2024, 1, 16, 3, 23, 47)
    """
    timestamp_element = element.find("timestamp")
    if timestamp_element is None:
        raise WeatherForecastParseError("timestamp tag in issue tag not found")

    timestamp = timestamp_element.text
    if timestamp is None:
        raise WeatherForecastParseError("timestamp tag in issue tag has no text")

    return datetime.strptime(timestamp, "%Y%m%d%H%M%S")



from dataclasses import dataclass

__all__ = ["Shakemap"]


@dataclass(slots=True)
class Shakemap:
    """
    A schema used to store info about shakemap.

    Attributes:
        file_name: name of shakemap file.
    """

    file_name: str



from .bmkg import BMKG
from .earthquake import Earthquake
from .shakemap import Shakemap
from .weather_forecast import WeatherForecast

__all__ = [
    "BMKG",
    "Earthquake",
    "Shakemap",
    "WeatherForecast",
]



from dataclasses import dataclass

from .earthquake import Earthquake
from .shakemap import Shakemap

__all__ = ["LatestEarthquake"]


@dataclass(slots=True)
class LatestEarthquake:
    """
    A schema used to store info about latest earthquake.

    Attributes:
        earthquake: earthquake schema.
        potency: potential tsunami or not, and the status of the earthquake felt.
        felt: area that felt an earthquake on the MMI scale.
        shakemap: shakemap schema.
    """

    earthquake: Earthquake
    potency: str
    felt: str
    shakemap: Shakemap



from collections.abc import Iterator

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError
from ..schemas import WindSpeed

__all__ = ["parse_wind_speed_element"]


def parse_wind_speed_element(element: Element) -> Iterator[WindSpeed]:
    """
    Parse wind speed element tree in xml.

    Args:
        element: a parameter element that contain wind speed

    Yields:
        Some `WindSpeed` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<parameter id="ws" description="Wind speed" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="Kt">5</value>'
    ...     '<value unit="MPH">5.75389725</value>'
    ...     '<value unit="KPH">9.26</value>'
    ...     '<value unit="MS">2.57222222</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="Kt">2</value>'
    ...     '<value unit="MPH">2.3015589</value>'
    ...     '<value unit="KPH">3.704</value>'
    ...     '<value unit="MS">1.028888888</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="Kt">0</value>'
    ...     '<value unit="MPH">0</value>'
    ...     '<value unit="KPH">0</value>'
    ...     '<value unit="MS">0</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> wind_speed = parse_wind_speed_element(element)
    >>> wind_speed
    <generator object parse_wind_speed_element at ...>
    """
    for timerange in element:
        value_elements = timerange.findall("value")
        if len(value_elements) < 4:
            raise WeatherForecastParseError(
                "one or more value tag in timerange tag not found"
            )

        knot = value_elements[0].text
        if knot is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        mph = value_elements[1].text
        if mph is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        kph = value_elements[2].text
        if kph is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        ms = value_elements[3].text
        if ms is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield WindSpeed(float(knot), float(mph), float(kph), float(ms))



"""
Module to store common enum classes used by bmkg.
"""
from .cardinal import Cardinal
from .province import Province
from .sexa import Sexa
from .type import Type
from .weather import Weather

__all__ = [
    "Cardinal",
    "Province",
    "Sexa",
    "Type",
    "Weather",
]



from dataclasses import dataclass, field

__all__ = ["WindSpeed"]


@dataclass(slots=True)
class WindSpeed:
    """
    A schema used to store info about wind speed.

    Attributes:
        knot: knot of a wind speed with kn as its unit.
        mph: mph of a wind speed with mph as its unit.
        kph: kph of a wind speed with km/h as its unit.
        ms: ms of a wind speed with m/s as its unit.
    """

    knot: float = field(metadata={"unit": "kn"})
    mph: float = field(metadata={"unit": "mph"})
    kph: float = field(metadata={"unit": "km/h"})
    ms: float = field(metadata={"unit": "m/s"})



from dataclasses import dataclass

from .earthquake import Earthquake

__all__ = ["FeltEarthquake"]


@dataclass(slots=True)
class FeltEarthquake:
    """
    A schema used to store info about felt earthquake.

    Attributes:
        earthquake: earthquake schema.
        felt: area that felt an earthquake on the MMI scale.
    """

    earthquake: Earthquake
    felt: str



from collections.abc import Iterator
from itertools import chain

# FIXME
# remove `type: ignore` if there is a stub for defusedxml
from defusedxml.ElementTree import fromstring  # type: ignore

from ..enums import Type
from ..exceptions import WeatherForecastParseError
from ..schemas import (
    Weather,
    WeatherForecast,
)
from ..types import (
    WeatherForecastParameter,
    WeatherForecastParameterId,
    WeatherForecastParameters,
)
from .parse_area_element import parse_area_element
from .parse_data_element import parse_data_element
from .parse_datetime_element import parse_datetime_element
from .parse_forecast_element import parse_forecast_element
from .parse_humidity_element import parse_humidity_element
from .parse_issue_element import parse_issue_element
from .parse_temperature_element import parse_temperature_element
from .parse_weather_element import parse_weather_element
from .parse_wind_direction_element import parse_wind_direction_element
from .parse_wind_speed_element import parse_wind_speed_element

__all__ = ["parse_weather_forecast_data"]


def parse_weather_forecast_data(weather_forecast_data: str | bytes) -> WeatherForecast:
    """
    Parse weather forecast data element tree in xml.

    This control when area type is sea then it has no information regarding weather
    forecast. Also this use wind speed element tree to get the datetime information.

    Args:
        weather_forecast_data: string or bytes of xml data.

    Returns:
        A` WeatherForecast` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.
    """
    root = fromstring(weather_forecast_data)

    data = parse_data_element(root)

    forecast_element = root[0]
    forecast = parse_forecast_element(forecast_element)

    issue_element = forecast_element[0]
    issue = parse_issue_element(issue_element)

    areas = {}
    for area_element in forecast_element.iterfind("area"):
        area = parse_area_element(area_element)
        # if the area type is sea, then the weather is empty
        weathers: Iterator[Weather] | None = None

        # Only land that has weather, sea doesn't
        if area.type == Type.LAND:
            parameters: WeatherForecastParameters = {}
            for parameter_element in area_element.iterfind("parameter"):
                parameter_id: WeatherForecastParameterId | None = parameter_element.get(
                    "id"
                )
                if parameter_id is None:
                    raise WeatherForecastParseError(
                        "id attribute in parameter tag not found"
                    )

                parameter: WeatherForecastParameter
                if (
                    parameter_id == "hu"
                    or parameter_id == "humax"
                    or parameter_id == "humin"
                ):
                    parameter = parse_humidity_element(parameter_element)
                elif (
                    parameter_id == "t"
                    or parameter_id == "tmax"
                    or parameter_id == "tmin"
                ):
                    parameter = parse_temperature_element(parameter_element)
                elif parameter_id == "weather":
                    parameter = parse_weather_element(parameter_element)
                elif parameter_id == "wd":
                    parameter = parse_wind_direction_element(parameter_element)
                else:
                    # parameter_id == "ws":
                    parameter = parse_wind_speed_element(parameter_element)

                    # Choose one of parameter tag that has datetime attribute with
                    # type hourly. Strictly speaking it is timerange tag.
                    parameters["datetime"] = parse_datetime_element(parameter_element)

                parameters[parameter_id] = parameter

            # The length of the data to be zipped is 12. This is because the weather
            # forecast data received is for three days, each receiving 4 data. However,
            # for tmin, tmax, hmin, and hmax data, these four data are only given 1 each
            # for that day, therefore there are only 3 data for 3 days from tmin, tmax,
            # hmin, and hmax. Because the zip() function will stop at the smallest data
            # length, therefore we need to multiply the 3 data by 4 to get 12
            weathers = (
                Weather(dt, weather, t, tmix, tmax, h, hmin, hmax, wd, ws)
                for dt, weather, t, tmix, tmax, h, hmin, hmax, wd, ws in zip(
                    parameters["datetime"],
                    parameters["weather"],
                    parameters["t"],
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["tmin"]]
                    ),
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["tmax"]]
                    ),
                    parameters["hu"],
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["humin"]]
                    ),
                    chain.from_iterable(
                        [(val, val, val, val) for val in parameters["humax"]]
                    ),
                    parameters["wd"],
                    parameters["ws"],
                )
            )

        areas[area] = weathers

    return WeatherForecast(data, forecast, issue, areas)



from aiohttp import ClientSession

from .. import schemas
from .api import API

__all__ = ["Shakemap"]


class Shakemap(API, schemas.Shakemap):
    """
    Shakemap API Wrapper from BMKG API.
    """

    url = "/DataMKG/TEWS"

    def __init__(self, session: ClientSession, file_name: str) -> None:
        API.__init__(self, session)
        schemas.Shakemap.__init__(self, file_name)

    async def get_content(self) -> bytes:
        """
        Get the shakemap file content.

        Returns:
            A bytes of `Shakemap` image.

        Examples:
            >>> import asyncio
            >>> from bmkg import BMKG
            >>> async def main():
            ...     async with BMKG() as bmkg:
            ...         latest_earthquake = await bmkg.earthquake.get_latest_earthquake()
            ...         shakemap = latest_earthquake.shakemap
            ...         shakemap_content = await shakemap.get_content()
            ...         print(shakemap_content)
            >>> asyncio.run(main())
            b'...'
        """  # noqa: E501
        async with self._session.get(f"{self.url}/{self.file_name}") as response:
            return await response.read()



from collections.abc import Iterator

from .. import schemas
from ..parsers import (
    parse_felt_earthquake_data,
    parse_latest_earthquake_data,
    parse_strong_earthquake_data,
)
from .api import API
from .shakemap import Shakemap

__all__ = ["Earthquake"]


class Earthquake(API):
    """
    Earthquake API Wrapper from BMKG API.
    """

    url = "/DataMKG/TEWS"

    async def get_latest_earthquake(self) -> schemas.LatestEarthquake:
        """
        Request latest earthquake from earthquake API.

        Returns:
            A `LatestEarthquake` schema.

        Examples:
            >>> import asyncio
            >>> from bmkg import BMKG
            >>> async def main():
            ...     async with BMKG() as bmkg:
            ...         latest_earthquake = await bmkg.earthquake.get_latest_earthquake()
            ...         print(latest_earthquake)
            >>> asyncio.run(main())
            LatestEarthquake(earthquake=Earthquake(datetime=datetime.datetime(...)

        Notes:
            The `LatestEarthquake` schema has a `shakemap` field which is the `Shakemap` API.
        """  # noqa: E501
        async with self._session.get(f"{self.url}/autogempa.json") as response:
            latest_earthquake = parse_latest_earthquake_data(await response.json())  # type: ignore

            latest_earthquake.shakemap = Shakemap(
                self._session, latest_earthquake.shakemap.file_name
            )

            return latest_earthquake

    async def get_strong_earthquake(self) -> Iterator[schemas.StrongEarthquake]:
        """
        Request strong earthquake that has magnitude 5.0 above from earthquake API.

        Returns:
            An iterator of fifteen `StrongEarthquake` schema.

        Examples:
            >>> import asyncio
            >>> from bmkg import BMKG
            >>> async def main():
            ...     async with BMKG() as bmkg:
            ...         strong_earthquake = await bmkg.earthquake.get_strong_earthquake()
            ...         print(strong_earthquake)
            >>> asyncio.run(main())
            <generator object parse_strong_earthquake_data at ...>
        """  # noqa: E501
        async with self._session.get(f"{self.url}/gempaterkini.json") as response:
            return parse_strong_earthquake_data(await response.json())  # type: ignore

    async def get_felt_earthquake(self) -> Iterator[schemas.FeltEarthquake]:
        """
        Request felt earthquake from earthquake API.

        Returns:
            An iterator of fifteen `FeltEarthquake` schema.

        Examples:
            >>> import asyncio
            >>> from bmkg import BMKG
            >>> async def main():
            ...     async with BMKG() as bmkg:
            ...         felt_earthquake = await bmkg.earthquake.get_felt_earthquake()
            ...         print(felt_earthquake)
            >>> asyncio.run(main())
            <generator object parse_felt_earthquake_data at ...>
        """
        async with self._session.get(f"{self.url}/gempadirasakan.json") as response:
            return parse_felt_earthquake_data(await response.json())  # type: ignore



"""
Module to store a str enum class representation the name of the province.
"""
from enum import StrEnum

__all__ = ["Province"]


class Province(StrEnum):
    """
    A str enum class that define valid province.

    Attributes:
        ACEH: `"Aceh"`
        BALI: `"Bali"`
        BANGKA_BELITUNG: `"BangkaBelitung"`
        BANTEN: `"Banten"`
        BENGKULU: `"Bengkulu"`
        DI_YOGYAKARTA: `"DIYogyakarta"`
        DKI_JAKARTA: `"DKIJakarta"`
        GORONTALO: `"Gorontalo"`
        JAMBI: `"Jambi"`
        JAWA_BARAT: `"JawaBarat"`
        JAWA_TENGAH: `"JawaTengah"`
        JAWA_TIMUR: `"JawaTimur"`
        KALIMANTAN_BARAT: `"KalimantanBarat"`
        KALIMANTAN_SELATAN: `"KalimantanSelatan"`
        KALIMANTAN_TENGAH: `"KalimantanTengah"`
        KALIMANTAN_TIMUR: `"KalimantanTimur"`
        KALIMANTAN_UTARA: `"KalimantanUtara"`
        KEPULAUAN_RIAU: `"KepulauanRiau"`
        LAMPUNG: `"Lampung"`
        MALUKU: `"Maluku"`
        MALUKU_UTARA: `"MalukuUtara"`
        NUSA_TENGGARA_BARAT: `"NusaTenggaraBarat"`
        NUSA_TENGGARA_TIMUR: `"NusaTenggaraTimur"`
        PAPUA: `"Papua"`
        PAPUA_BARAT: `"PapuaBarat"`
        RIAU: `"Riau"`
        SULAWESI_BARAT: `"SulawesiBarat"`
        SULAWESI_SELATAN: `"SulawesiSelatan"`
        SULAWESI_TENGAH: `"SulawesiTengah"`
        SULAWESI_TENGGARA: `"SulawesiTenggara"`
        SULAWESI_UTARA: `"SulawesiUtara"`
        SUMATERA_BARAT: `"SumateraBarat"`
        SUMATERA_SELATAN: `"SumateraSelatan"`
        SUMATERA_UTARA: `"SumateraUtara"`
        INDONESIA: `"Indonesia"`

    Examples:
        >>> Province("Aceh")
        <Province.ACEH: 'Aceh'>
        >>> Province["ACEH"]
        <Province.ACEH: 'Aceh'>
        >>> Province.ACEH
        <Province.ACEH: 'Aceh'>
        >>> Province.ACEH == "Aceh"
        True
        >>> print(Province.ACEH)
        Aceh

    Note:
        `INDONESIA` is the country name.
    """

    ACEH: str = "Aceh"
    BALI: str = "Bali"
    BANGKA_BELITUNG: str = "BangkaBelitung"
    BANTEN: str = "Banten"
    BENGKULU: str = "Bengkulu"
    DI_YOGYAKARTA: str = "DIYogyakarta"
    DKI_JAKARTA: str = "DKIJakarta"
    GORONTALO: str = "Gorontalo"
    JAMBI: str = "Jambi"
    JAWA_BARAT: str = "JawaBarat"
    JAWA_TENGAH: str = "JawaTengah"
    JAWA_TIMUR: str = "JawaTimur"
    KALIMANTAN_BARAT: str = "KalimantanBarat"
    KALIMANTAN_SELATAN: str = "KalimantanSelatan"
    KALIMANTAN_TENGAH: str = "KalimantanTengah"
    KALIMANTAN_TIMUR: str = "KalimantanTimur"
    KALIMANTAN_UTARA: str = "KalimantanUtara"
    KEPULAUAN_RIAU: str = "KepulauanRiau"
    LAMPUNG: str = "Lampung"
    MALUKU: str = "Maluku"
    MALUKU_UTARA: str = "MalukuUtara"
    NUSA_TENGGARA_BARAT: str = "NusaTenggaraBarat"
    NUSA_TENGGARA_TIMUR: str = "NusaTenggaraTimur"
    PAPUA: str = "Papua"
    PAPUA_BARAT: str = "PapuaBarat"
    RIAU: str = "Riau"
    SULAWESI_BARAT: str = "SulawesiBarat"
    SULAWESI_SELATAN: str = "SulawesiSelatan"
    SULAWESI_TENGAH: str = "SulawesiTengah"
    SULAWESI_TENGGARA: str = "SulawesiTenggara"
    SULAWESI_UTARA: str = "SulawesiUtara"
    SUMATERA_BARAT: str = "SumateraBarat"
    SUMATERA_SELATAN: str = "SumateraSelatan"
    SUMATERA_UTARA: str = "SumateraUtara"
    INDONESIA: str = "Indonesia"



from collections.abc import Iterator

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from .. import enums
from ..exceptions import WeatherForecastParseError

__all__ = ["parse_weather_element"]


def parse_weather_element(element: Element) -> Iterator[enums.Weather]:
    """
    Parse weather element tree in xml.

    Args:
        element: a parameter element that contain weather

    Yields:
        Some `Weather` enum.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<parameter id="weather" description="Weather" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="icon">60</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="icon">60</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="icon">1</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> weather = parse_weather_element(element)
    >>> weather
    <generator object parse_weather_element at ...>
    """
    for timerange in element:
        value_elements = timerange.find("value")
        if value_elements is None:
            raise WeatherForecastParseError("value tag in timerange tag not found")

        weather = value_elements.text
        if weather is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield enums.Weather(int(weather))



from collections.abc import Iterator

from ..schemas import StrongEarthquake
from ..types import InfoStrongEarthquakeData
from .parse_earthquake_data import parse_earthquake_data

__all__ = ["parse_strong_earthquake_data"]


def parse_strong_earthquake_data(
    info_strong_earthquake_data: InfoStrongEarthquakeData,
) -> Iterator[StrongEarthquake]:
    """
    Parse `InfoStrongEarthquakeData` JSON.

    Args:
        info_strong_earthquake_data: A dict representing JSON of info strong earthquake.

    Yields:
        A `StrongEarthquake` schema.

    Examples:
        >>> info_strong_earthquake_data = InfoStrongEarthquakeData(
        ...     {
        ...         "Infogempa": {
        ...             "gempa": [
        ...                 {
        ...                     "Tanggal": "09 Jan 2024",
        ...                     "Jam": "12:25:24 WIB",
        ...                     "DateTime": "2024-01-09T05:25:24+00:00",
        ...                     "Coordinates": "-4.46,133.91",
        ...                     "Lintang": "4.46 LS",
        ...                     "Bujur": "133.91 BT",
        ...                     "Magnitude": "5.3",
        ...                     "Kedalaman": "10 km",
        ...                     "Wilayah": "130 km BaratDaya KAIMANA-PAPUABRT",
        ...                     "Potensi": "Tidak berpotensi tsunami",
        ...                 }
        ...             ]
        ...         }
        ...     }
        ... )
        >>> parse_strong_earthquake_data(info_strong_earthquake_data)
        <generator object parse_strong_earthquake_data at ...>
    """

    for strong_earthquake_data in info_strong_earthquake_data["Infogempa"]["gempa"]:
        earthquake = parse_earthquake_data(strong_earthquake_data)
        potency = strong_earthquake_data["Potensi"]

        yield StrongEarthquake(earthquake, potency)



from collections.abc import Iterator
from datetime import datetime

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError

__all__ = ["parse_datetime_element"]


def parse_datetime_element(element: Element) -> Iterator[datetime]:
    """
    Parse datetime element tree in xml.

    This parse datetime string found in element tree in the following format
    `"%Y%m%d%H%M%S"`.

    Args:
        element: any parameter element that contain datetime with type hourly

    Yields:
        Some `datetime` object.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<parameter id="ws" description="datetime" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="Kt">5</value>'
    ...     '<value unit="MPH">5.75389725</value>'
    ...     '<value unit="KPH">9.26</value>'
    ...     '<value unit="MS">2.57222222</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="Kt">2</value>'
    ...     '<value unit="MPH">2.3015589</value>'
    ...     '<value unit="KPH">3.704</value>'
    ...     '<value unit="MS">1.028888888</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="Kt">0</value>'
    ...     '<value unit="MPH">0</value>'
    ...     '<value unit="KPH">0</value>'
    ...     '<value unit="MS">0</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> datetime = parse_datetime_element(element)
    >>> datetime
    <generator object parse_datetime_element at ...>
    """
    for timerange in element:
        dt = timerange.get("datetime")
        if dt is None:
            raise WeatherForecastParseError(
                "datetime attribute in timerange tag not found"
            )

        yield datetime.strptime(dt, "%Y%m%d%H%M%S")



from collections.abc import Iterator

from ..schemas import FeltEarthquake
from ..types import InfoFeltEarthquakeData
from .parse_earthquake_data import parse_earthquake_data

__all__ = ["parse_felt_earthquake_data"]


def parse_felt_earthquake_data(
    info_felt_earthquake_data: InfoFeltEarthquakeData,
) -> Iterator[FeltEarthquake]:
    """
    Parse `InfoFeltEarthquakeData` JSON.

    Args:
        info_felt_earthquake_data: A dict representing JSON of info felt earthquake.

    Yields:
        A `FeltEarthquake` schema.

    Examples:
        >>> info_felt_earthquake_data = InfoFeltEarthquakeData(
        ...     {
        ...         "Infogempa": {
        ...             "gempa": [
        ...                 {
        ...                     "Tanggal": "15 Jan 2024",
        ...                     "Jam": "16:07:08 WIB",
        ...                     "DateTime": "2024-01-15T09:07:08+00:00",
        ...                     "Coordinates": "-3.98,122.45",
        ...                     "Lintang": "3.98 LS",
        ...                     "Bujur": "122.45 BT",
        ...                     "Magnitude": "2.6",
        ...                     "Kedalaman": "9 km",
        ...                     "Wilayah": "Pusat gempa berada di darat 2,5 km BaratDa",
        ...                     "Dirasakan": "II Kendari",
        ...                 }
        ...             ]
        ...         }
        ...     }
        ... )
        >>> parse_felt_earthquake_data(info_felt_earthquake_data)
        <generator object parse_felt_earthquake_data at ...>
    """

    for felt_earthquake_data in info_felt_earthquake_data["Infogempa"]["gempa"]:
        earthquake = parse_earthquake_data(felt_earthquake_data)
        felt = felt_earthquake_data["Dirasakan"]

        yield FeltEarthquake(earthquake, felt)



from dataclasses import dataclass
from datetime import datetime

from ..enums import Weather as WeatherEnum
from .humidity import Humidity
from .temperature import Temperature
from .wind_direction import WindDirection
from .wind_speed import WindSpeed

__all__ = ["Weather"]


@dataclass(slots=True)
class Weather:
    """
    A schema used to store info about weather.

    Attributes:
        datetime: datetime of a weather.
        weather: `Weather` enum symbolic names (members) of a weather condition.
        temperature: temperature of a weather.
        minimum_temperature: minimum temperature of a weather.
        maximum_temperature: maximum temperature of a weather.
        humidity: humidity of a weather.
        min_humidity: minimum humidity of a weather.
        max_humidity: maximum humidity of a weather.
        wind_direction: wind direction of a weather.
        wind_speed: wind speed of a weather.

    Note:
        `datetime` is naive datetime, means it has no information about its timezone.
        Also don't be confused with `weather`, this `weather` field is weather enum that
        has representation of weather condition.
    """

    datetime: datetime
    weather: WeatherEnum
    temperature: Temperature
    minimum_temperature: Temperature
    maximum_temperature: Temperature
    humidity: Humidity
    min_humidity: Humidity
    max_humidity: Humidity
    wind_direction: WindDirection
    wind_speed: WindSpeed



from .parse_area_element import parse_area_element
from .parse_data_element import parse_data_element
from .parse_datetime_element import parse_datetime_element
from .parse_earthquake_data import parse_earthquake_data
from .parse_felt_earthquake_data import parse_felt_earthquake_data
from .parse_forecast_element import parse_forecast_element
from .parse_humidity_element import parse_humidity_element
from .parse_issue_element import parse_issue_element
from .parse_latest_earthquake_data import parse_latest_earthquake_data
from .parse_strong_earthquake_data import parse_strong_earthquake_data
from .parse_temperature_element import parse_temperature_element
from .parse_weather_element import parse_weather_element
from .parse_weather_forecast_data import parse_weather_forecast_data
from .parse_wind_direction_element import parse_wind_direction_element
from .parse_wind_speed_element import parse_wind_speed_element

__all__ = [
    "parse_earthquake_data",
    "parse_felt_earthquake_data",
    "parse_latest_earthquake_data",
    "parse_strong_earthquake_data",
    "parse_data_element",
    "parse_forecast_element",
    "parse_issue_element",
    "parse_area_element",
    "parse_humidity_element",
    "parse_temperature_element",
    "parse_weather_element",
    "parse_wind_direction_element",
    "parse_wind_speed_element",
    "parse_datetime_element",
    "parse_weather_forecast_data",
]



from datetime import datetime

from ..schemas import (
    Coordinate,
    Earthquake,
)
from ..types import EarthquakeData

__all__ = ["parse_earthquake_data"]


def parse_earthquake_data(earthquake_data: EarthquakeData) -> Earthquake:
    """
    Parse `EarthquakeData` JSON.

    Args:
        earthquake_data: A dict representing JSON of earthquake.

    Returns:
        An `Earthquake` schema.

    Examples:
        >>> earthquake_data = EarthquakeData(
        ...     {
        ...         "Tanggal": "14 Jan 2024",
        ...         "Jam": "03:13:37 WIB",
        ...         "DateTime": "2024-01-13T20:13:37+00:00",
        ...         "Coordinates": "-6.75,106.55",
        ...         "Lintang": "6.75 LS",
        ...         "Bujur": "106.55 BT",
        ...         "Magnitude": "3.5",
        ...         "Kedalaman": "5 km",
        ...         "Wilayah": "Pusat gempa berada di darat 26 km Utara Kab Sukabumi",
        ...     }
        ... )
        >>> parse_earthquake_data(earthquake_data)
        Earthquake(datetime=datetime.datetime(2024, 1, 13, 20, 13, 37, tzinfo=datetim...
    """

    dt = earthquake_data["DateTime"]
    coordinate = earthquake_data["Coordinates"].split(",")
    latitude = coordinate[0]
    longitude = coordinate[1]
    magnitude = earthquake_data["Magnitude"]
    depth = earthquake_data["Kedalaman"].split()[0]
    region = earthquake_data["Wilayah"]

    return Earthquake(
        datetime.fromisoformat(dt),
        Coordinate(float(latitude), float(longitude)),
        float(magnitude),
        float(depth),
        region,
    )



from collections.abc import Iterator

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..enums import Cardinal, Sexa
from ..exceptions import WeatherForecastParseError
from ..schemas import WindDirection

__all__ = ["parse_wind_direction_element"]


def parse_wind_direction_element(element: Element) -> Iterator[WindDirection]:
    """
    Parse wind direction element tree in xml.

    Args:
        element: a parameter element that contain wind direction

    Yields:
        Some `WindDirection` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<parameter id="wd" description="Wind direction" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="deg">90</value>'
    ...     '<value unit="CARD">E</value>'
    ...     '<value unit="SEXA">9000</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="deg">157.5</value>'
    ...     '<value unit="CARD">SSE</value>'
    ...     '<value unit="SEXA">15730</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="deg">0</value>'
    ...     '<value unit="CARD">VARIABLE</value>'
    ...     '<value unit="SEXA">000</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> wind_direction = parse_wind_direction_element(element)
    >>> wind_direction
    <generator object parse_wind_direction_element at ...>
    """
    for timerange in element:
        value_elements = timerange.findall("value")
        if len(value_elements) < 3:
            raise WeatherForecastParseError(
                "one or more value tag in timerange tag not found"
            )

        deg = value_elements[0].text
        if deg is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        card = value_elements[1].text
        if card is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        sexa = value_elements[2].text
        if sexa is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield WindDirection(float(deg), Cardinal(card), Sexa(sexa))



# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError
from ..schemas import Forecast

__all__ = ["parse_forecast_element"]


def parse_forecast_element(element: Element) -> Forecast:
    """
    Parse forecast element tree in xml.

    Args:
        element: an element of forecast

    Returns:
        A `Forecast` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring('<forecast domain="local"></forecast>')
    >>> forecast = parse_forecast_element(element)
    >>> forecast
    Forecast(domain='local')
    """
    domain = element.get("domain")
    if domain is None:
        raise WeatherForecastParseError("domain attribute in forecast tag not found")

    return Forecast(domain)



from .area import Area
from .coordinate import Coordinate
from .data import Data
from .earthquake import Earthquake
from .felt_earthquake import FeltEarthquake
from .forecast import Forecast
from .humidity import Humidity
from .latest_earthquake import LatestEarthquake
from .name import Name
from .shakemap import Shakemap
from .strong_earthquake import StrongEarthquake
from .temperature import Temperature
from .weather import Weather
from .weather_forecast import WeatherForecast
from .wind_direction import WindDirection
from .wind_speed import WindSpeed

__all__ = [
    "Area",
    "Coordinate",
    "Data",
    "Earthquake",
    "FeltEarthquake",
    "Forecast",
    "Humidity",
    "LatestEarthquake",
    "Name",
    "Shakemap",
    "StrongEarthquake",
    "Temperature",
    "WeatherForecast",
    "Weather",
    "WindDirection",
    "WindSpeed",
]



from dataclasses import dataclass

from ..enums import Type
from .coordinate import Coordinate
from .name import Name

__all__ = ["Area"]


@dataclass(unsafe_hash=True, slots=True)
class Area:
    """
    A schema used to store info about area.

    Attributes:
        id: id of an area.
        coordinate: coordinate of an area.
        type: type of an area.
        region: region of an area.
        level: level of an area.
        description: description of an area.
        domain: domain of an area.
        tags: tags of an area.
        names: names of an area.
    """

    id: str
    coordinate: Coordinate
    type: Type
    region: str
    level: str
    description: str
    domain: str
    tags: str
    names: Name



# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..enums import Type
from ..exceptions import WeatherForecastParseError
from ..schemas import (
    Area,
    Coordinate,
    Name,
)

__all__ = ["parse_area_element"]


def parse_area_element(element: Element) -> Area:
    """
    Parse area element tree in xml.

    Args:
        element: an element of area

    Returns:
        An `Area` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     "<area"
    ...     ' id="501409"'
    ...     ' latitude="4.176594"'
    ...     ' longitude="96.124878"'
    ...     ' coordinate="96.124878 4.176594"'
    ...     ' type="land"'
    ...     ' region=""'
    ...     ' level="1"'
    ...     ' description="Aceh Barat"'
    ...     ' domain="Aceh"'
    ...     ' tags=""'
    ...     ">"
    ...     '<name xml:lang="en_US">Aceh Barat</name>'
    ...     '<name xml:lang="id_ID">Kab. Aceh Barat</name>'
    ...     "</area>"
    ... )
    >>> area = parse_area_element(element)
    >>> area
    Area(id='501409', coordinate=Coordinate(latitude=4.176594, longitude=96.124878), ...
    """
    id = element.get("id")
    if id is None:
        raise WeatherForecastParseError("id attribute in area tag not found")

    latitude = element.get("latitude")
    if latitude is None:
        raise WeatherForecastParseError("latitude attribute in area tag not found")

    longitude = element.get("longitude")
    if longitude is None:
        raise WeatherForecastParseError("longitude attribute in area tag not found")

    type = element.get("type")
    if type is None:
        raise WeatherForecastParseError("type attribute in area tag not found")

    region = element.get("region")
    if region is None:
        raise WeatherForecastParseError("region attribute in area tag not found")

    level = element.get("level")
    if level is None:
        raise WeatherForecastParseError("level attribute in area tag not found")

    description = element.get("description")
    if description is None:
        raise WeatherForecastParseError("description attribute in area tag not found")

    domain = element.get("domain")
    if domain is None:
        raise WeatherForecastParseError("domain attribute in area tag not found")

    tags = element.get("tags")
    if tags is None:
        raise WeatherForecastParseError("tags attribute in area tag not found")

    names = element.findall("name")
    if len(names) < 2:
        raise WeatherForecastParseError("one or more name tag in area tag not found")

    en_US = names[0].text
    if en_US is None:
        raise WeatherForecastParseError("name tag in area tag has no text")

    id_ID = names[1].text
    if id_ID is None:
        raise WeatherForecastParseError("name tag in area tag has no text")

    name = Name(en_US, id_ID)

    return Area(
        id,
        Coordinate(float(latitude), float(longitude)),
        Type(type),
        region,
        level,
        description,
        domain,
        tags,
        name,
    )



# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError
from ..schemas import Data

__all__ = ["parse_data_element"]


def parse_data_element(element: Element) -> Data:
    """
    Parse data element tree in xml.

    Args:
        element: an element of data

    Returns:
        A `Data` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<data source="meteofactory" productioncenter="NC Jakarta"></data>'
    ... )
    >>> data = parse_data_element(element)
    >>> data
    Data(source='meteofactory', productioncenter='NC Jakarta')
    """
    source = element.get("source")
    if source is None:
        raise WeatherForecastParseError("source attribute in data tag not found")

    productioncenter = element.get("productioncenter")
    if productioncenter is None:
        raise WeatherForecastParseError(
            "productioncenter attribute in data tag not found"
        )

    return Data(source, productioncenter)



"""
Module to store a str enum class representation sexa directions.
"""
from enum import StrEnum

__all__ = ["Sexa"]


class Sexa(StrEnum):
    """
    A str enum class that define valid sexa direction.

    Attributes:
        NORTH_NORTHEAST: `"2230"`
        NORTHEAST: `"4500"`
        EAST_NORTHEAST: `"6730"`
        EAST: `"9000"`
        EAST_SOUTHEAST: `"11230"`
        SOUTHEAST: `"13500"`
        SOUTH_SOUTHEAST: `"15730"`
        SOUTH: `"18000"`
        SOUTH_SOUTHWEST: `"20230"`
        SOUTHWEST: `"22500"`
        WEST_SOUTHWEST: `"24730"`
        WEST: `"27000"`
        WEST_NORTHWEST: `"29230"`
        NORTHWEST: `"31500"`
        NORTH_NORTHWEST: `"33730"`
        VARIABLE: `"000"`

    Examples:
        >>> Sexa("2230")
        <Sexa.NORTH_NORTHEAST: '2230'>
        >>> Sexa["NORTH_NORTHEAST"]
        <Sexa.NORTH_NORTHEAST: '2230'>
        >>> Sexa.NORTH_NORTHEAST
        <Sexa.NORTH_NORTHEAST: '2230'>
        >>> Sexa.NORTH_NORTHEAST == "2230"
        True
        >>> print(Sexa.NORTH_NORTHEAST)
        2230

    Note:
        `VARIABLE` direction mean as it's name suggest, the direction can't be
        determined.

    Warning:
        There is no `NORTH` due to bug in BMKG API.
    """

    # FIXME
    # Change NORTH or VARIABLE if the bug was fixed
    # Possible bug in BMKG API that NORTH equal to VARIABLE
    # NORTH: str = "000"
    NORTH_NORTHEAST: str = "2230"
    NORTHEAST: str = "4500"
    EAST_NORTHEAST: str = "6730"
    EAST: str = "9000"
    EAST_SOUTHEAST: str = "11230"
    SOUTHEAST: str = "13500"
    SOUTH_SOUTHEAST: str = "15730"
    SOUTH: str = "18000"
    SOUTH_SOUTHWEST: str = "20230"
    SOUTHWEST: str = "22500"
    WEST_SOUTHWEST: str = "24730"
    WEST: str = "27000"
    WEST_NORTHWEST: str = "29230"
    NORTHWEST: str = "31500"
    NORTH_NORTHWEST: str = "33730"
    VARIABLE: str = "000"



from aiohttp import ClientSession

__all__ = ["API"]


class API:
    """
    Base API class.
    """

    def __init__(self, session: ClientSession | None = None) -> None:
        self._session = (
            session if session is not None else ClientSession("https://data.bmkg.go.id")
        )



from collections.abc import Iterator

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError
from ..schemas import Humidity

__all__ = ["parse_humidity_element"]


def parse_humidity_element(element: Element) -> Iterator[Humidity]:
    """
    Parse humidity element tree in xml.

    Args:
        element: a parameter element that contain humidity

    Yields:
        Some `Humidity` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<parameter id="hu" description="Humidity" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="%">95</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="%">90</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="%">95</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> humidity = parse_humidity_element(element)
    >>> humidity
    <generator object parse_humidity_element at ...>
    """
    for timerange in element:
        value_element = timerange.find("value")
        if value_element is None:
            raise WeatherForecastParseError(
                "value tag in timerange tag not found"
            ) from AttributeError

        humidity_text = value_element.text
        if humidity_text is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield Humidity(int(humidity_text))



from collections.abc import Iterator

# FIXME
# Element is used only for typing not parsing
from xml.etree.ElementTree import Element  # nosec B405

from ..exceptions import WeatherForecastParseError
from ..schemas import Temperature

__all__ = ["parse_temperature_element"]


def parse_temperature_element(element: Element) -> Iterator[Temperature]:
    """
    Parse temperature element tree in xml.

    Args:
        element: a parameter element that contain temperature

    Yields:
        Some `Temperature` schema.

    Raises:
        WeatherForecastParseError: If some expected attribute is not found.

    Examples:
    >>> from defusedxml.ElementTree import fromstring
    >>> element = fromstring(
    ...     '<parameter id="t" description="Temperature" type="hourly">'
    ...     '<timerange type="hourly" h="0" datetime="202401170000">'
    ...     '<value unit="C">24</value>'
    ...     '<value unit="F">75.2</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="6" datetime="202401170600">'
    ...     '<value unit="C">28</value>'
    ...     '<value unit="F">82.4</value>'
    ...     "</timerange>"
    ...     '<timerange type="hourly" h="12" datetime="202401171200">'
    ...     '<value unit="C">26</value>'
    ...     '<value unit="F">78.8</value>'
    ...     "</timerange>"
    ...     "</parameter>"
    ... )
    >>> temperature = parse_temperature_element(element)
    >>> temperature
    <generator object parse_temperature_element at ...>
    """
    for timerange in element:
        value_elements = timerange.findall("value")
        if len(value_elements) < 2:
            raise WeatherForecastParseError(
                "one or more value tag in timerange tag not found"
            )

        celcius = value_elements[0].text
        if celcius is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        fahrenheit = value_elements[1].text
        if fahrenheit is None:
            raise WeatherForecastParseError("value tag in timerange tag has no text")

        yield Temperature(float(celcius), float(fahrenheit))



from traceback import TracebackException
from types import TracebackType
from typing import Self

from .api import API
from .earthquake import Earthquake
from .weather_forecast import WeatherForecast

__all__ = ["BMKG"]


class BMKG(API):
    """
    Base BMKG API wrapper.

    Attributes:
        earthquake (Earthquake): earthquake api interface.
        weather_forecast (WeatherForecast): weather forecast api interface.
    """

    def __init__(self) -> None:
        API.__init__(self)

        self.earthquake = Earthquake(self._session)
        self.weather_forecast = WeatherForecast(self._session)

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(
        self,
        exc_type: Exception,
        exc_val: TracebackException,
        traceback: TracebackType,
    ) -> None:
        await self.close()

    async def close(self) -> None:
        await self._session.close()



__all__ = [
    "BMKGError",
    "WeatherForecastParseError",
]


class BMKGError(Exception):
    """
    General BMKG API exception.
    """

    pass


class WeatherForecastParseError(BMKGError):
    """
    Weather forecast parse exception.
    """

    pass
